<?php
/**
 * Admin Settings class
 */

namespace Parcelbroker\Admin;
use Parcelbroker;

class Settings extends Parcelbroker\Admin\Core{

	//protected $parcelbrokerwc_surcharge_list = [72,80,82,83,87,89,92,93,95,102,103,105,174,175,176];
	//protected $parcelbrokerwc_dropoff = [178,162];

	/**
	 * Class constructor
	 * Add various actions
	 */
	public function __construct()
	{
		if (NO_WOO) {
			add_action('admin_notices', array($this, 'woocommerceNotice'));
			return;
		}
		add_action('admin_notices', array($this, 'registerNotice'));
		add_action('admin_enqueue_scripts', array($this, 'enqueue_scripts'));
		add_action('admin_enqueue_scripts', array($this, 'enqueue_styles'));
		add_action('admin_menu', array($this, 'setupPluginSettings'), 90);
		add_action('admin_init', array($this, 'registerSettings'));
		add_action('wp_ajax_parcelbrokerwc_dismiss_notice', array($this, 'dismissNotice'));

	}

	/**
	 * Enqueue styles
	 */
	public function enqueue_styles()
	{
		wp_enqueue_style("parcelbroker", WP_PLUGIN_URL.'/parcelbroker-for-woocommerce/css/style.css', []);
	}

	/**
	 * Dismiss notice action
	 */
	public function dismissNotice()
	{
		update_option('parcelbrokerwc-dismiss_notice', true);
	}

	/**
	 * Enqueue scripts
	 */
	public function enqueue_scripts()
	{
		wp_register_script( "parcelbrokerwc-admin", WP_PLUGIN_URL.'/parcelbroker-for-woocommerce/js/admin.js', array('jquery') );
		wp_enqueue_script( 'parcelbrokerwc-admin' );
	}

	/**
	 * Display Woocommerce requirement notification
	 */
	public function woocommerceNotice()
	{
		$message = __('The ParcelBroker plugin requires <a href="https://woocommerce.com/" target="_BLANK">Woocommerce</a> to be installed and active');
		echo "<div class=\"error notice\">
		    <p>{$message}</p>
		</div>";
	}

	/**
	 * Display API settings notification
	 */
	public function registerNotice()
	{
		if($this->isRegistered() || get_option($this->parcelbrokerwc_settings['dismiss_notice'])) {
			return;
		}

		$message = __('You need to register your API credentials before the ParcelBroker plugin will work. <a href="' .admin_url('admin.php?page=parcelbroker-settings'). '">Settings</a>');
		echo "<div class=\"notice-warning notice is-dismissible parcelbrokerwc-notice\">
		    <p>{$message}</p>
		</div>";
	}

	/**
	 * Register settings and setting blocks
	 */
	public function registerSettings()
	{
		$settings_group = 'parcelbrokerwc';
		foreach($this->parcelbrokerwc_settings as $setting) {
			register_setting($settings_group, $setting);
		}

		$settings_section = 'parcelbrokerwc-settings';
		$page = $settings_group;

		add_settings_section(
			$settings_section . '-api',
			__('ParcelBroker API settings'),
			array($this, 'settingApiSectionRender'),
			$page
		);

		add_settings_section(
			$settings_section . '-shipping',
			__('ParcelBroker Shipping settings'),
			null,
			$page
		);

		add_settings_section(
			$settings_section . '-collection',
			__('ParcelBroker Collection Address'),
			function() { _e('This will default to your WooCommerce shop address'); },
			$page
		);

		add_settings_section(
			$settings_section . '-tracking',
			__('ParcelBroker Tracking settings'),
			null,
			$page
		);

		add_settings_field(
			$this->parcelbrokerwc_settings['api_key'],
			__('Api key'),
			array($this, 'settingApiKeyFieldRender'),
			$page,
			$settings_section . '-api'
		);

		add_settings_field(
			$this->parcelbrokerwc_settings['api_token'],
			__('Api token'),
			array($this, 'settingApiTokenFieldRender'),
			$page,
			$settings_section . '-api'
		);

		add_settings_field(
			$this->parcelbrokerwc_settings['use_shipping'],
			__('Book shipping via ParcelBroker'),
			array($this, 'settingUseShippingFieldRender'),
			$page,
			$settings_section . '-shipping'
		);

		add_settings_field(
			$this->parcelbrokerwc_settings['use_shipping_dimensions'],
			__('Use packaged dimensions'),
			array($this, 'settingUseShippingDimensionsFieldRender'),
			$page,
			$settings_section . '-shipping'
		);

		add_settings_field(
			$this->parcelbrokerwc_settings['use_test_shipping'],
			__('Enable Test Mode'),
			array($this, 'settingUseTestShippingFieldRender'),
			$page,
			$settings_section . '-shipping'
		);

		/* ========================= */
		add_settings_field(
			$this->parcelbrokerwc_settings['collection_name'],
			__('Name'),
			array($this, 'settingCollectionNameFieldRender'),
			$page,
			$settings_section . '-collection'
		);

		add_settings_field(
			$this->parcelbrokerwc_settings['collection_company'],
			__('Company'),
			array($this, 'settingCollectionCompanyFieldRender'),
			$page,
			$settings_section . '-collection'
		);

		add_settings_field(
			$this->parcelbrokerwc_settings['collection_address_1'],
			__('Address Line 1'),
			array($this, 'settingCollectionAddressLine1FieldRender'),
			$page,
			$settings_section . '-collection'
		);

		add_settings_field(
			$this->parcelbrokerwc_settings['collection_address_2'],
			__('Address Line 2'),
			array($this, 'settingCollectionAddressLine2FieldRender'),
			$page,
			$settings_section . '-collection'
		);

		add_settings_field(
			$this->parcelbrokerwc_settings['collection_town'],
			__('Town'),
			array($this, 'settingCollectionTownFieldRender'),
			$page,
			$settings_section . '-collection'
		);

		add_settings_field(
			$this->parcelbrokerwc_settings['collection_state'],
			__('County/State'),
			array($this, 'settingCollectionStateFieldRender'),
			$page,
			$settings_section . '-collection'
		);

		add_settings_field(
			$this->parcelbrokerwc_settings['collection_zip'],
			__('Postcode/Zipcode'),
			array($this, 'settingCollectionZipFieldRender'),
			$page,
			$settings_section . '-collection'
		);

		add_settings_field(
			$this->parcelbrokerwc_settings['collection_country'],
			__('Country'),
			array($this, 'settingCollectionCountryFieldRender'),
			$page,
			$settings_section . '-collection'
		);

		add_settings_field(
			$this->parcelbrokerwc_settings['collection_phone'],
			__('Phone number'),
			array($this, 'settingCollectionPhoneFieldRender'),
			$page,
			$settings_section . '-collection'
		);

		add_settings_field(
			$this->parcelbrokerwc_settings['collection_email'],
			__('Email'),
			array($this, 'settingCollectionEmailFieldRender'),
			$page,
			$settings_section . '-collection'
		);


		/* ========================= */

		add_settings_field(
			$this->parcelbrokerwc_settings['powered_by'],
			__('Display powered by ParcelBroker'),
			array($this, 'settingPoweredByFieldRender'),
			$page,
			$settings_section . '-tracking'
		);

	}

	/**
	 * Api section setting description
	 */
	public function settingApiSectionRender()
	{
		_e('Contact <a href="https://parcelbroker.co.uk/contact/" target="_BLANK">ParcelBroker</a> to arrange access to your API details');
	}

	/**
	 * Api key setting input
	 */
	public function settingApiKeyFieldRender()
	{
		$option = get_option($this->parcelbrokerwc_settings['api_key']);
		echo "<input type='text' name='{$this->parcelbrokerwc_settings['api_key']}' value='{$option}'>";
	}

	/**
	 * Api token setting input
	 */
	public function settingApiTokenFieldRender()
	{
		$option = get_option($this->parcelbrokerwc_settings['api_token']);
		echo "<input type='text' name='{$this->parcelbrokerwc_settings['api_token']}' value='{$option}'>";
	}

	/**
	 * Use shipping setting input
	 */
	public function settingUseShippingFieldRender()
	{
		$option = get_option($this->parcelbrokerwc_settings['use_shipping']);
		$checked = $option ? ' checked':'';
		echo "<input type='checkbox' name='{$this->parcelbrokerwc_settings['use_shipping']}'{$checked} value='1'>";
	}

	/**
	 * Use shipping dimensions setting input
	 */
	public function settingUseShippingDimensionsFieldRender()
	{
		$option = get_option($this->parcelbrokerwc_settings['use_shipping_dimensions']);
		$checked = $option ? ' checked':'';
		echo "<input type='checkbox' name='{$this->parcelbrokerwc_settings['use_shipping_dimensions']}'{$checked} value='1'>";
	}

	/**
	 * Use test setting input
	 */
	public function settingUseTestShippingFieldRender()
	{
		$option = get_option($this->parcelbrokerwc_settings['use_test_shipping']);
		$checked = $option ? ' checked':'';
		echo "<input type='checkbox' name='{$this->parcelbrokerwc_settings['use_test_shipping']}'{$checked} value='1'>";
	}

 	/**
 	 * Collection name setting input
 	 */
	public function settingCollectionNameFieldRender()
	{
		$option = get_option($this->parcelbrokerwc_settings['collection_name']);
		echo "<input type='text' name='{$this->parcelbrokerwc_settings['collection_name']}' value='{$option}'>";
	}

 	/**
 	 * Collection company setting input
 	 */
	public function settingCollectionCompanyFieldRender()
	{
		$option = get_option($this->parcelbrokerwc_settings['collection_company']);
		echo "<input type='text' name='{$this->parcelbrokerwc_settings['collection_company']}' value='{$option}'>";
	}

 	/**
 	 * Collection address line 1 setting input
 	 */
	public function settingCollectionAddressLine1FieldRender()
	{
		$option = get_option($this->parcelbrokerwc_settings['collection_address_1']);
		echo "<input type='text' name='{$this->parcelbrokerwc_settings['collection_address_1']}' value='{$option}'>";
	}

 	/**
 	 * Collection address line 2 setting input
 	 */
	public function settingCollectionAddressLine2FieldRender()
	{
		$option = get_option($this->parcelbrokerwc_settings['collection_address_2']);
		echo "<input type='text' name='{$this->parcelbrokerwc_settings['collection_address_2']}' value='{$option}'>";
	}

 	/**
 	 * Collection town setting input
 	 */
	public function settingCollectionTownFieldRender()
	{
		$option = get_option($this->parcelbrokerwc_settings['collection_town']);
		echo "<input type='text' name='{$this->parcelbrokerwc_settings['collection_town']}' value='{$option}'>";
	}

 	/**
 	 * Collection state setting input
 	 */
	public function settingCollectionStateFieldRender()
	{
		$option = get_option($this->parcelbrokerwc_settings['collection_state']);
		echo "<input type='text' name='{$this->parcelbrokerwc_settings['collection_state']}' value='{$option}'>";
	}

 	/**
 	 * Collection zip setting input
 	 */
	public function settingCollectionZipFieldRender()
	{
		$option = get_option($this->parcelbrokerwc_settings['collection_zip']);
		echo "<input type='text' name='{$this->parcelbrokerwc_settings['collection_zip']}' value='{$option}'>";
	}

 	/**
 	 * Collection country setting input
 	 */
	public function settingCollectionCountryFieldRender()
	{
		$option = get_option($this->parcelbrokerwc_settings['collection_country']);
		//echo "<input type='text' name='{$this->parcelbrokerwc_settings['collection_country']}' value='{$option}'>";
		$countries_obj   = new \WC_Countries();
	    $countries   = $countries_obj->__get('countries');
		woocommerce_form_field($this->parcelbrokerwc_settings['collection_country'], array(
			'type'        => 'country',
		    'default'	  => $option,
		));
	}

 	/**
 	 * Collection phone setting input
 	 */
	public function settingCollectionPhoneFieldRender()
	{
		$option = get_option($this->parcelbrokerwc_settings['collection_phone']);
		echo "<input type='text' name='{$this->parcelbrokerwc_settings['collection_phone']}' value='{$option}'>";
	}

 	/**
 	 * Collection email setting input
 	 */
	public function settingCollectionEmailFieldRender()
	{
		$option = get_option($this->parcelbrokerwc_settings['collection_email']);
		echo "<input type='text' name='{$this->parcelbrokerwc_settings['collection_email']}' value='{$option}'>";
	}

 	/**
 	 * Powered by ParcelBroker setting input
 	 */
	public function settingPoweredByFieldRender()
	{
		$option = get_option($this->parcelbrokerwc_settings['powered_by']);
		$checked = $option ? ' checked':'';
		echo "<input type='checkbox' name='{$this->parcelbrokerwc_settings['powered_by']}'{$checked} value='1'>";
	}

	/**
	 * Add setting submenu page to woocommerce
	 */
	public function setupPluginSettings()
	{
		add_submenu_page(
			'woocommerce',
			'ParcelBroker Settings',
			'ParcelBroker Settings',
			'manage_options',
			'parcelbroker-settings',
			array($this, 'pluginSettings')
		);
	}

	/**
	 * Settings page render
	 */
	public function pluginSettings()
	{
		// Check that the user is allowed to update options
		if (!current_user_can('manage_options')) {
		    wp_die('You do not have sufficient permissions to access this page.');
		}
		$messages = get_settings_errors();
		$flash = '';
		foreach ($messages as $message) {
			$flash .= "<div class=\"notice {$message['type']} is-dismissible\"><p>{$message['message']}</p></div>";
		}
?>

		<div>
			<h1 class="wp-heading-inline"><?php _e('ParcelBroker settings') ?></h1>
			<hr class="wp-header-end">
			<?php echo $flash ?>
			<form method="post" action="options.php">
				<?php
					\settings_fields('parcelbrokerwc');
					\do_settings_sections('parcelbrokerwc');
					\submit_button();
				?>
			</form>
		</div>

<?php
	}
}