<?php
/**
 * Admin Shipping class
 */

namespace Parcelbroker\Admin;
use Parcelbroker;

class Shipping extends Parcelbroker\Admin\Core{
	use \ParcelbrokerApiTrait;
	/**
	 * Class constructor
	 * Add actions, filters and load country
	 */
	public function __construct()
	{
		add_action('woocommerce_privacy_before_remove_order_personal_data', array($this, 'privacy_remove_order_personal_data_props'));
		if (!get_option($this->parcelbrokerwc_settings['use_shipping'], false) || NO_WOO) {
			return;
		}
		$this->loadCountry();

		if($this->isRegistered()) {
			add_action('admin_enqueue_scripts', array($this, 'enqueue_scripts'));
			add_action('add_meta_boxes', array($this, 'renderMetabox'));
			add_action('save_post_shop_order', array($this, 'bookCollection'), 1, 3);
			add_action('save_post_product', array($this, 'saveProduct'), 10, 3);

			if(get_option($this->parcelbrokerwc_settings['use_shipping_dimensions'], false)) {
				add_action('woocommerce_product_options_dimensions', array($this, 'packageDimensions'));
			}

			add_action('wp_ajax_parcelbrokerwc_fetch_pricing', array($this, 'fetchPricing'));
			add_action('wp_ajax_parcelbrokerwc_update_state_postcode', array($this, 'updateStatePostcode'));
			add_action( 'admin_notices', array($this, 'shippingErrorNotice') );

			add_action('admin_menu', array($this, 'setupPluginErrorReportingTools'), 50);
	    	add_filter('post_updated_messages', [$this, 'updatePostSuccessMessage'], 11);
		}
	}

	/**
	 * Enqueue scripts
	 */
	public function enqueue_scripts()
	{
		global $post;
		if ( !$post || $post->post_type !== 'shop_order' ) return;

		wp_register_script( "parcelbrokerwc-shipping-admin", WP_PLUGIN_URL.'/parcelbroker-for-woocommerce/js/shipping-admin.js', array('jquery') );
		wp_localize_script( 'parcelbrokerwc-shipping-admin', 'pbShippingAdmin', array(
			'addParcel' => array(
				'parcelWeight' => __('Parcel Weight', 'parcelbrokerwc'),
				'parcelHeight' => __('Parcel Height', 'parcelbrokerwc'),
				'parcelWidth' => __('Parcel Width', 'parcelbrokerwc'),
				'parcelLength' => __('Parcel Length', 'parcelbrokerwc'),
				'qty' => __('Quantity', 'parcelbrokerwc'),
				'serviceError' => __('Could not retreive any services', 'parcelbrokerwc')
			),
			'surchargeRates' => $this->parcelbrokerwc_surcharge_list,
			'dropoff' => $this->parcelbrokerwc_dropoff,
			'samedayCollection' => $this->parcelbrokerwc_samedaycollection,
			'bookingConfirmation' => __('Are you sure you want to create another booking for this order?', 'pbwc')
		));

		wp_enqueue_script( 'parcelbrokerwc-shipping-admin' );
	}

	/**
	 * Create the meta box on the order page
	 */
	public function renderMetabox()
	{
		global $post;

		if(get_post_meta($post->ID, '_anonymized', null)) {
			return;
		}

		add_meta_box('parcelbroker-book-collection', 'ParcelBroker Book Collection', array( $this, 'metaboxContent'), 'shop_order');
	}

	/**
	 * Meta box content for shipping
	 */
	public function metaboxContent()
	{
		global $post, $thepostid, $theorder;

		if ( ! is_int( $thepostid ) ) {
			$thepostid = $post->ID;
		}

		if ( ! is_object( $theorder ) ) {
			$theorder = wc_get_order( $thepostid );
		}

		$order_response = get_post_meta($thepostid, '_parcelbrokerwc-order_response', true);

		$countries = WC()->countries->get_countries();

		// Get devliery details
		$delivery = $theorder->get_address('shipping');
		$delivery_email = $theorder->get_billing_email();
		$delivery_phone = $theorder->get_billing_phone();
		$delivery_states = WC()->countries->get_states($delivery['country']);
		$delivery_state = !empty($delivery_states) ? $delivery_states[$delivery['state']] : $delivery['state'];


		// Get shop/collection details
		$shop_contact = get_option($this->parcelbrokerwc_settings['collection_name'], null);
		$shop_company = get_option($this->parcelbrokerwc_settings['collection_company'], get_option('blogname', null));
		$shop_email = get_option($this->parcelbrokerwc_settings['collection_email'], get_option('admin_email', null));
		$shop_phone = get_option($this->parcelbrokerwc_settings['collection_phone'], null);
		$shop_address_1 = get_option($this->parcelbrokerwc_settings['collection_address_1'], get_option('woocommerce_store_address', null));
		$shop_address_2 = get_option($this->parcelbrokerwc_settings['collection_address_2'], get_option('woocommerce_store_address_2', null));
		$shop_town = get_option($this->parcelbrokerwc_settings['collection_town'], get_option('woocommerce_store_city', null));
		$shop_postcode = get_option($this->parcelbrokerwc_settings['collection_zip'], get_option('woocommerce_store_postcode', null));
		$default_country_state = explode(':', get_option('woocommerce_default_country', null));
		$shop_state = !empty($default_country_state[1]) ? $default_country_state[1] : null;
		$shop_state = get_option($this->parcelbrokerwc_settings['collection_state'], $shop_state);
		$shop_country = get_option($this->parcelbrokerwc_settings['collection_country'], $default_country_state[0]);

		// Build item list from order
	    $item_list = array();
	    $cat_list = array();
	    $shipping_dimensions = get_option($this->parcelbrokerwc_settings['use_shipping_dimensions']);
	    foreach( $theorder->get_items() as $item ) {
	    	$product = $item->get_product();
	    	if ($product->is_virtual() ) {
	    		continue;
	    	}

	    	foreach($product->category_ids as $cat_id) {
	    		$cat_name = get_the_category_by_ID($cat_id);
	    		if($cat_name != 'Uncategorised') {
	    			$cat_list[$cat_id] = $cat_name;
	    		}
	    	}

	    	$item_list = $this->addPackingItemList( array(
	    		'weight' => $product->get_weight(),
	    		'height' => $shipping_dimensions && $product->get_meta('_package_height') ?
	    						$product->get_meta('_package_height') : $product->get_height(),
	    		'width' =>  $shipping_dimensions && $product->get_meta('_package_width') ?
	    						$product->get_meta('_package_width') : $product->get_width(),
	    		'length' => $shipping_dimensions && $product->get_meta('_package_length') ?
	    						$product->get_meta('_package_length') : $product->get_length(),
	    		'qty' => $item['qty']
	    	), $item_list);

	    }
	    $shipping_contents = implode(',', $cat_list);
	    $error_count = get_post_meta($thepostid, '_parcelbrokerwc-errors_count', true);
	    $is_customs = get_post_meta($post->ID, '_parcelbrokerwc-use_customs', true);
	    $customs_message = '';
		if($is_customs != 'false') {
			$customs_message .= " and your customs documentation";
		}


	    //Get holiday listing from ParcelBroker
	    $natDates = file_get_contents('https://parcelbroker.co.uk/v2/api/date/restrictions');

	    // Build state and postcode fields based on country
	    $state_field = $this->getCollectionLocaleInput($shop_country, $shop_state, 'County/State', 'state', 'state');
	    $postcode_field = $this->getCollectionLocaleInput($shop_country, $shop_postcode, 'Postcode/Zip', 'postcode', 'text');

		include(WP_PLUGIN_DIR . '/parcelbroker-for-woocommerce/templates/order-shipping.php');
	}

	// Display extra fields for package dimensions
	public function packageDimensions()
	{
		global $product_object;
		include(WP_PLUGIN_DIR . '/parcelbroker-for-woocommerce/templates/parcelbrokerwc-product-data-shipping-dimensions.php');
	}

	// Save extra product dimensions on product save
	public function saveProduct($post_id, $post, $update)
	{
		if(get_option($this->parcelbrokerwc_settings['use_shipping_dimensions'], false)) {
			update_post_meta($post_id, '_package_length', $_POST['_package_length']);
			update_post_meta($post_id, '_package_width', $_POST['_package_width']);
			update_post_meta($post_id, '_package_height', $_POST['_package_height']);
		}

	}

	/**
	 * Add item to array and calculate whether dimensions and weight already exist in the collection
	 * @param Array $item_data  Data for the item to be inserted
	 * @param Array $item_array Current item list
	 * @return Array	Newly modified item list
	 */
	private function addPackingItemList($item_data, $item_array)
	{
		$matching_id = null;
		foreach($item_array as $k => $item) {
			if(
				!empty($item) && !empty($item_data) &&
				$item['weight'] == $item_data['weight'] &&
				$item['height'] == $item_data['height'] &&
				$item['length'] == $item_data['length'] &&
				$item['width'] == $item_data['width']
			) {
				$matching_id = $k;
			}
		}
		if( $matching_id !== null ) {
			$item_array[$matching_id]['qty'] += $item_data['qty'];
		} else {
			$item_array[] = $item_data;
		}


		return $item_array;
	}

	/**
	 * Ajax function to fetch carrier pricing
	 * @return JSON API response
	 */
	public function fetchPricing()
	{
		$item_list = [];
		$lines = 0;
		$url_item_arr = [];
		foreach($_POST['parcels'] as $parcel) {
			for($i = 0; $i < $parcel['qty']; $i++) {
				$item_list[] = array(
					'weight' => $parcel['weight'],
					'height' => $parcel['height'],
					'length' => $parcel['length'],
					'width' => $parcel['width']
				);
				$url_item_arr[] = 'weight_' . $lines . '=' . $parcel['weight'] . '&height_' . $lines . '=' . $parcel['height'] . '&length_' . $lines . '=' . $parcel['length'] . '&width_' . $lines . '=' . $parcel['width'];
				$lines++;
			}
			}

		if((strpos($_POST['collection']['countrycode'], 'UK') === false && $_POST['collection']['countrycode'] != 'GB') || (strpos($_POST['delivery']['countrycode'], 'UK') === false) && $_POST['delivery']['countrycode'] != 'GB') {

			$url = 'https://parcelbroker.co.uk/?type=PARCEL&uom_w=kg&oum_l=cm&lines=' . $lines . '&' . implode('&', $url_item_arr) . '&from=' . $_POST['collection']['countrycode'] . '&to=' . $_POST['delivery']['countrycode'] . '&c_postcode=' . $_POST['collection']['postcode'] . '&d_postcode=' . $_POST['delivery']['postcode'] . '&bp=1';
			$error = 'International shipments are currently disabled, here is a quick link to get a quote directly on our <a target="_BLANK" href="' . $url . '">site</a>';
			wp_send_json(['error' => $error]);
		}

		$xml_response = $this->getApiShipping($_POST['collection'], $_POST['delivery'], $item_list);

		if($xml_response['header']['status'] == 'Error') {
			if(get_option($this->parcelbrokerwc_settings['use_test_shipping'])) {
				return wp_send_json($xml_response['errors']);
			}
			return array();
		}
		wp_send_json($xml_response['services']);
	}

	/**
	 * Make booking collection API call
	 * @param  int $post_id Order id
	 * @param  Object $post    Order object
	 * @param  bool $update
	 */
	public function bookCollection($post_id, $post, $update)
	{
		global $theorder;

		if ( ! is_object( $theorder ) ) {
			$theorder = wc_get_order( $post_id );
		}

		// Only continue if the book collection has been fired
		if ( !$_POST['pbwc-book-collection'] ) return;

		// Make API call
		$return = $this->sendApiCollection($_POST['pbwc-shipping'], $theorder, $_POST['pbwc-collection'], $_POST['pbwc-delivery'], $_POST['pbwc-parcel'], $_POST['pbwc-shipping-contents'], $_POST['pbwc-customs'], $_POST['packaging'], $_POST['pbwc-colletion-date'], $_POST['pbwc-collection-date']);

		$response = $return['response'];
		// Save API response
	    $theorder->update_meta_data( '_parcelbrokerwc-order_response', $response);
	    $theorder->update_meta_data( '_parcelbrokerwc-use_customs', $_POST['pbwc-customs']['use']);
	    $theorder->save();
		if ( $response['errors'] ) {
			// Save data if booking errored
			$theorder->update_meta_data( '_parcelbrokerwc-order_errors', $response['errors']);
			$error_count = get_post_meta($post_id, '_parcelbrokerwc-errors_count', true)?:0;
			$theorder->update_meta_data( '_parcelbrokerwc-errors_count',  $error_count + 1);
		    $theorder->update_meta_data( '_parcelbrokerwc-parcel_list', $return['parcels'] );
		    $theorder->update_meta_data( '_parcelbrokerwc-xml', $return['xml'] );
	   		$theorder->save();
			wp_redirect( admin_url( '/post.php?action=edit&post=' . $post_id ), 301 );
        	exit;
		}

	    if( $response['shipment'] ) {
	    	$theorder->update_meta_data( 'bookSuccess', true);
	    	// Store tracking number
	    	$theorder->update_meta_data( $this->parcelbrokerwc_tracking_key, $response['shipment']['ordernumber']);
	    	// null uneeded data
			$theorder->update_meta_data( '_parcelbrokerwc-errors_count',  0);
		    $theorder->update_meta_data( '_parcelbrokerwc-parcel_list', null );
			$theorder->update_meta_data( '_parcelbrokerwc-order_errors', null);
	    	$theorder->save();
	    }
	}

	/**
	 * Show shippign error notice if a booking failed
	 * @return [type] [description]
	 */
	public function shippingErrorNotice()
	{
		global $post;
		if(!$post) return;
		if($post->ID && get_post_meta($post->ID, '_parcelbrokerwc-order_errors')) {
			delete_post_meta($post->ID, '_parcelbrokerwc-order_errors');
			echo '<div class="notice error" >';
			echo '<p>' . __('There was an error booking the collection. Please try again or contact ParcelBroker', 'parcelbrokerwc') . '</p>';
		    echo '</div>';
		}
	}

	/**
	 * Ajax function to update the state and postcode fields on the fly depending on country
	 * @return JSON
	 */
	public function updateStatePostcode()
	{
		$country_code = $_POST['countrycode'];
		$state = $_POST['state'];
		$postcode = $_POST['postcode'];

	    $state_field = $this->getCollectionLocaleInput($country_code, $state, 'County/State', 'state', 'state');
	    $postcode_field = $this->getCollectionLocaleInput($country_code, $postcode, 'Postcode/Zip', 'postcode', 'text');

		wp_send_json(['state' => $state_field, 'postcode' => $postcode_field]);
	}

	/**
	 * Return woocommerce inputs validating with locales
	 * @param  String $country_code Country code
	 * @param  String $default      Default value to fill the input with
	 * @param  String $label        Default label for the input
	 * @param  string $field        Field name
	 * @param  string $type         Input Type
	 * @return String               HTML content for woocommerce input
	 */
	private function getCollectionLocaleInput($country_code, $default, $label, $field = 'state', $type = 'state')
	{
		$locales = WC()->countries->get_country_locale();
		$selected_locale = $locales[$country_code];
		$required = true;
		if(!empty($selected_locale)){
			if(!empty($selected_locale[$field]) && !empty($selected_locale[$field]['label'])) {
				$label = $selected_locale[$field]['label'];
			}
			if(!empty($selected_locale[$field]) && isset($selected_locale[$field]['required'])) {
				$required = $selected_locale[$field]['required'];
			}
		}

		return woocommerce_form_field('pbwc-collection['.$field.']', array(
		    'type'        => $type,
		    'required'    => $required,
		    'return' 	  => true,
		    'default'	  => $default,
		    'country'	  => $country_code,
		    'label'       => __('Collection ', 'parcelbrokerwc') . $label,
		    'custom_attributes' => array('data-parcelbrokerwc-collection' => $field, 'style' => 'width: 95%')
		));
	}


	/*********** ERROR REPORTING *************/

	/**
	 * Add submenu for the error reporting screen
	 */
	public function setupPluginErrorReportingTools()
	{
		add_submenu_page(
			'tools.php',
			'ParcelBroker Report Error',
			'ParcelBroker Report Error',
			'manage_options',
			'parcelbroker-report-error',
			array($this, 'pluginErrorReporting')
		);
	}

	/**
	 * Render Error reporting screen
	 */
	public function pluginErrorReporting()
	{

		// Submit error via email
		if(isset($_POST['pbwc-submit-error']) && $order = get_post($_GET['pbwcOrder'])) {
			$note = $_POST['pbwc-error-notes'];
			$message =
				"\n\n--- PARCEL LIST ---\n\n" .
				json_encode(get_post_meta($order->ID, '_parcelbrokerwc-parcel_list', true)) .
				"\n\n--- XML REQUEST ---\n\n" .
				get_post_meta($order->ID, '_parcelbrokerwc-xml', true) .
				"\n\n--- ORDER RESPONSE ---\n\n" .
				json_encode(get_post_meta($order->ID, '_parcelbrokerwc-order_response', true)) .
				"\n\n--- NOTE ---\n\n" .
				$note .
				"\n\n--- API KEY ---\n\n" .
				get_option($this->parcelbrokerwc_settings['api_key']);
				wp_mail('it@parcelbroker.co.uk', 'Wordpress Plugin Error Report', $message);

			update_post_meta($order->ID, '_parcelbrokerwc-error-mail', $message);
			update_post_meta($order->ID, '_parcelbrokerwc-error-date', time());
			wp_redirect( admin_url( '/tools.php?page=parcelbroker-report-error' ) );
		}

		// Display single error page
		if($order = get_post($_GET['pbwcOrder'])) {
			if(get_post_meta($order->ID, '_parcelbrokerwc-errors_count', true)) {
				include(WP_PLUGIN_DIR . '/parcelbroker-for-woocommerce/templates/parcelbrokerwc-shipping-error-report.php');
			} else {
				wp_redirect( admin_url( '/tools.php?page=parcelbroker-report-error' ) );
			}
		} else { // Show list of errors reported
			$orders_query = new \WP_Query( array(
				'post_type' => 'shop_order',
				'post_status' => 'all',
				'meta_query' => array(
					'relation' => 'AND',
					array(
						'key' => '_parcelbrokerwc-errors_count',
            			'compare' => 'EXISTS',
					),
					array(
						'key' => $this->parcelbrokerwc_tracking_key,
						'compare' => 'NOT EXISTS'
					)
				)
			));
			include(WP_PLUGIN_DIR . '/parcelbroker-for-woocommerce/templates/parcelbrokerwc-shipping-error-list.php');
		}
	}

	/**
	 * Hook into woocommerce delete personal data to remove extra stored data
	 * @param  Object $order Order object
	 */
	public function privacy_remove_order_personal_data_props($order)
	{
		delete_post_meta($order->ID, '_parcelbrokerwc-error-mail');
		delete_post_meta($order->ID, '_parcelbrokerwc-error-date');
		delete_post_meta($order->ID, '_parcelbrokerwc-order_response');
		delete_post_meta($order->ID, '_parcelbrokerwc-order_errors');
		delete_post_meta($order->ID, '_parcelbrokerwc-errors_count');
		delete_post_meta($order->ID, '_parcelbrokerwc-parcel_list');
		delete_post_meta($order->ID, '_parcelbrokerwc-xml');
		delete_post_meta($order->ID, '_parcelbrokerwc-use_customs');
	}

	public function updatePostSuccessMessage($messages) {
		global $post;
	    $book_success = get_post_meta($post->ID,  'bookSuccess', true);
	    $is_customs = get_post_meta($post->ID, '_parcelbrokerwc-use_customs', true);
	    $customs_message = '';
		delete_post_meta($post->ID, 'bookSuccess');
		if ( $book_success ) {
			if($is_customs != 'false') {
				$customs_message .= " and your customs documentation";
			}
			$messages['shop_order'][4] = $messages['shop_order'][1] . sprintf(__(' You will receive your order label via email, alternatively you can go to your <a href="%s" target="_blank">account</a> on ParcelBroker and download the label'.$customs_message.' from there.', 'parcelbrokerwc'), 'https://parcelbroker.co.uk/account/myorders.php');
		}
		return $messages;
	}

}