<?php
/**
 * Admin Tracking class
 */

namespace Parcelbroker\Admin;
use Parcelbroker;

class Tracking extends Parcelbroker\Admin\Core{

	/**
	 * Class constructor
	 * Add actions
	 */
	public function __construct()
	{
		if (NO_WOO) {
			return;
		}
		if($this->isRegistered()) {
			add_action('add_meta_boxes', array($this, 'renderMetabox'));
			add_action('save_post_shop_order', array($this, 'saveOrder'), 10, 3);
			add_action('wp_ajax_get_tracking', array($this, 'getTracking'));
			add_action('woocommerce_email_customer_details', array($this, 'orderEmail'), 50, 4);
		}
	}

	/**
	 * Create the meta box on the order page
	 */
	public function renderMetabox()
	{
		add_meta_box('parcelbroker', 'ParcelBroker', array( $this, 'metaboxContent'), 'shop_order', 'side');
	}

	/**
	 * Meta box content for shipping
	 */
	public function metaboxContent()
	{
		echo '<p>Tracking Number</p><p><input value="' . $this->getPostTracking() . '" class="input-text" name="_pb_tracking_reference"></p>';
	}

	// Save extra product dimensions on product save
	public function saveOrder($post_id, $post, $update)
	{
		if ( $_POST['pbwc-book-collection'] ) return;
		update_post_meta($post_id, $this->parcelbrokerwc_tracking_key, $_POST['_pb_tracking_reference']);
	}

	/**
	 * Get tracking AJAX function
	 */
	public function getTracking()
	{
		$is_error = !$this->isRegistered();
		$tracking_ref = $_REQUEST['tracking_number'];
		if(!$is_error && false === ($xml_response = get_transient('parcelbrokerwc-' . $tracking_ref))) {
			$xml_response = $this->getApiTracking($tracking_ref);
			$template = '';

		}
		if($is_error || !empty($xml_response->errors)) {
			$error = "<tr><td class=\"parcelbrokerwc-error\">" . __('Error getting tracking') . "</td></tr>";
			$template = apply_filters('parcelbroker-tracking-error', $error);
		} else {
			set_transient('parcelbrokerwc-' . $tracking_ref, $xml_response, 15 * MINUTE_IN_SECONDS);
			$tracking = $xml_response['tracking'];
			$date_format = get_option('date_format', 'd-m-Y');
			$time_format = get_option('time_format', 'H:i');
			ob_start();
			if(empty($tracking['track'])) {
				$description = $tracking['status'];
				$hide_top = true;
				include($this->getTemplate('template-tracking'));
				$template = ob_get_contents() . $template;
				ob_clean();
			} else {
				$hide_top = false;
				foreach($tracking['track'] as $track){
					$track_data = (array)$track;
					$datetime = date($date_format . ' ' . $time_format, strtotime($track_data['@attributes']['datetime']));

					$status = apply_filters('parcelbrokerwc-tracking-status', $track_data['status']);
					$description = apply_filters('parcelbrokerwc-tracking-description', $track_data['description']);

					include($this->getTemplate('template-tracking'));
					$template = ob_get_contents() . $template;
					ob_clean();
				}
			}
			ob_end_clean();

		}
		$template_start = apply_filters('parcelbrokerwc_tracking_template_start', '<table class="parcelbrokerwc-tracking">');
		$template_end = apply_filters('parcelbrokerwc_tracking_template_end', '</table>');
		$template = $template_start . $template . $template_end;
		if(get_option('parcelbrokerwc-powered_by')) {
			$template .= "<p>".__("Tracking powered by <a href=\"https://parcelbroker.co.uk\">ParcelBroker</a>")."</p>";
		}
		echo json_encode(['tracking' => $xml_response, 'output' => $template]);
		die;
	}

	/**
	 * Add Tracking reference to the woocommerce emails
	 * @param  Object $order         Order object
	 * @param  Bool $sent_to_admin Is the email being sent to the admin
	 * @param  string $plain_text
	 * @param  string $email         Email address being sent to
	 */
	public function orderEmail($order, $sent_to_admin, $plain_text, $email)
	{
		$tracking_ref = get_post_meta($order->get_id(), $this->parcelbrokerwc_tracking_key, true);
		if(!empty($tracking_ref) && !$sent_to_admin) {
			echo "<p style=\"margin: 0 0 16px;\">Your tracking reference is: {$tracking_ref}</p>";
		}
	}

}