<?php

namespace Parcelbroker;

class Parcelbroker {
	use \ParcelbrokerApiTrait;
	protected $version = '1.0.2';

	/**
	 * Filters to handle country adding
	 */
	protected function loadCountry()
	{
		add_filter('woocommerce_countries', array($this, 'addCountries'));
		add_filter('woocommerce_continents', array($this, 'addCountriesToContinents'));
		add_filter('woocommerce_get_country_locale', array($this, 'addCounrtyLocale'));
	}

	/**
	 * Front end public template
	 * @param  String $name Template name
	 * @return String       Template path
	 */
	protected function getTemplate($name)
	{
		if(file_exists(get_stylesheet_directory() . '/woocommerce/parcelbroker/' . $name . '.php')) {
			return get_stylesheet_directory() . '/woocommerce/parcelbroker-for-woocommerce/' . $name . '.php';
		} else if(file_exists(get_stylesheet_directory() . '/parcelbroker-for-woocommerce/' . $name . '.php')) {
			return get_stylesheet_directory() . '/parcelbroker-for-woocommerce/' . $name . '.php';
		} else {
			return WP_PLUGIN_DIR . '/parcelbroker-for-woocommerce/templates/' . $name . '.php';
		}
	}

	/**
	 * Check apoi key and api tokens are present
	 * @return boolean
	 */
	protected function isRegistered()
	{
		return get_option($this->parcelbrokerwc_settings['api_key']) && get_option($this->parcelbrokerwc_settings['api_token']);
	}

	/**
	 * Change GB name to UK - Mainland, add parcelbroker explicit coutries to break down the UK
	 * @param Array $countries
	 */
	public function addCountries($countries)
	{
		// Change additional countries to custom 2 letter ISO codes, add code to change them to appropriate codes for getting a quote and booking.
		return $countries;
		// unset($countries['GB']);
		// return array_merge(
		// 	$countries,
		// 	[
		// 		'GB' => 'UK - Mainland',
		// 		'UK_NI' => 'UK - Northern Ireland',
		// 		'UK_SH' => 'UK - Scottish Highlands & Irelands',
		// 		'UK_IOW' => 'UK - Isle of White',
		// 		'UK_IMIS' => 'UK - Isle of Man & Isle of Scilly'
		// 	]
		// );
	}

	/**
	 * Add new countries to appropriate continents
	 * @param Arrau $continents
	 */
	public function addCountriesToContinents($continents)
	{
		$continents['EU']['countries'][] = 'UK_NI';
		$continents['EU']['countries'][] = 'UK_SH';
		$continents['EU']['countries'][] = 'UK_IOW';
		$continents['EU']['countries'][] = 'UK_IMIS';
		return $continents;
	}

	/**
	 * Set new country locale information
	 * @param Array $locale
	 */
	public function addCounrtyLocale($locale)
	{
		$locale['UK_NI'] = array(
			'postcode' => array(
				'label' => __( 'Postcode', 'woocommerce' ),
			),
			'state'    => array(
				'label'    => __( 'County', 'woocommerce' ),
				'required' => false,
			)
		);
		$locale['UK_SH'] = array(
			'postcode' => array(
				'label' => __( 'Postcode', 'woocommerce' ),
			),
			'state'    => array(
				'label'    => __( 'County', 'woocommerce' ),
				'required' => false,
			)
		);
		$locale['UK_IOW'] = array(
			'postcode' => array(
				'label' => __( 'Postcode', 'woocommerce' ),
			),
			'state'    => array(
				'label'    => __( 'County', 'woocommerce' ),
				'required' => false,
			)
		);
		$locale['UK_IMIS'] = array(
			'postcode' => array(
				'label' => __( 'Postcode', 'woocommerce' ),
			),
			'state'    => array(
				'label'    => __( 'County', 'woocommerce' ),
				'required' => false,
			)
		);
		return $locale;
	}

}