<?php
/**
 * Front end parcel tracking class
 * Handles front end parcel tracking functionality and requirements
 */
namespace Parcelbroker;

class Tracking extends Parcelbroker {

	/**
	 * Class constructor
	 * Loads Country list modifications
	 * Adds actions
	 */
	public function __construct()
	{
		if(NO_WOO) {
			return;
		}
		$this->loadCountry();
		add_action('woocommerce_view_order', array($this, 'showOrderTrackingButton'), 5);
		add_action('wp_enqueue_scripts', array($this, 'enqueue_scripts'));
		add_action('wp_enqueue_scripts', array($this, 'enqueue_styles'));

	}

	/**
	 * Script enqueue
	 */
	public function enqueue_scripts()
	{
	   wp_register_script( "parcelbrokerTracking", WP_PLUGIN_URL.'/parcelbroker-for-woocommerce/js/tracking.js', array('jquery') );
	   wp_localize_script( 'parcelbrokerTracking', 'pb', array( 'ajaxurl' => admin_url( 'admin-ajax.php' )));
	   wp_enqueue_script( 'parcelbrokerTracking' );
	}

	/**
	 * Style enqueue
	 */
	public function enqueue_styles()
	{
		wp_enqueue_style("parcelbroker", WP_PLUGIN_URL.'/parcelbroker-for-woocommerce/css/style.css', []);
	}

	/**
	 * Inserts Show order tracking button into the order details page
	 * @param  int $order_id Current order id
	 */
	public function showOrderTrackingButton($order_id)
	{
		$tracking_number = get_post_meta($order_id, $this->parcelbrokerwc_tracking_key, true);

		// Only return the button if tracking number isn't empty and the plugin is registered
		if(empty($tracking_number) || !$this->isRegistered()) {
			return;
		}

		ob_start();
		include($this->getTemplate('view-tracking'));
		$out = ob_get_contents();
		ob_end_clean();
		echo $out;
	}

}